#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-03-31
#
# Monitors Cisco WLC temperature sensor
#
# tested with WLC2504, WLC5520
#
# 2016-07-02: fixed crit/warn to >=
# 2018-09-14: workaround for wrong data from Mobility Express Controller --> removed
# 2021-07-15: rewritten for CMK 2.0
#
# sample snmpwalk
#
# .1.3.6.1.4.1.14179.2.3.1.12.0 = STRING: "1"
# .1.3.6.1.4.1.14179.2.3.1.13.0 = STRING: "18"
# .1.3.6.1.4.1.14179.2.3.1.14.0 = STRING: "10"
# .1.3.6.1.4.1.14179.2.3.1.15.0 = STRING: "38"
#
# sample string_table
#
# [['1', '18', '10', '38']]
#

from dataclasses import dataclass
from typing import Optional
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    SNMPTree,
    contains,
    State,
    check_levels,
)


@dataclass
class CiscoWlcTemp:
    environment: str
    current: int
    lowlimit: float
    highlimit: float


_wlc_temp_environment = {
    '0': 'unknown',
    '1': 'commercial',
    '2': 'industrial',
}


def parse_cisco_wlc_temp(string_table: StringTable) -> Optional[CiscoWlcTemp]:
    try:
        environment, temperature, lowlimit, highlimit = string_table[0]
    except IndexError:
        return

    temperature = int(temperature)
    highlimit = int(highlimit)
    lowlimit = int(lowlimit)
    environment = _wlc_temp_environment.get(environment)
    return CiscoWlcTemp(
        environment=environment,
        current=temperature,
        lowlimit=lowlimit,
        highlimit=highlimit,
    )


def discovery_cisco_wlc_temp(section: CiscoWlcTemp) -> DiscoveryResult:
    yield Service()


def check_cisco_wlc_temp(params, section: CiscoWlcTemp) -> CheckResult:
    yield from check_levels(
        value=section.current,
        label='Temperature',
        levels_upper=params['levels_upper'],
        levels_lower=params['levels_lower'],
        render_func=lambda v: f'{v}°C',
        metric_name='temp',
        boundaries=(0, 100),
    )

    yield Result(state=State.OK,
                 notice=f'Lower/upper levels reported by the device: {section.lowlimit}°C/{section.highlimit}°C')
    yield Result(state=State.OK, notice=f'Environment type reported by the device: {section.environment}')


register.snmp_section(
    name='cisco_wlc_temp',
    parse_function=parse_cisco_wlc_temp,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.14179.2.3.1',  # AIRESPACE-WIRELESS-MIB::bsnGlobalDot11Config
        oids=[
            '12',  # bsnOperatingTemperatureEnvironment
            '13',  # bsnSensorTemperature (Current Internal Temperature of the unit in Centigrade)
            '14',  # bsnTemperatureAlarmLowLimit
            '15',  # bsnTemperatureAlarmHighLimit
        ]
    ),
    detect=contains('.1.3.6.1.2.1.1.1.0', 'Cisco Controller'),  # sysDescr
)

register.check_plugin(
    name='cisco_wlc_temp',
    service_name='Temperature',
    discovery_function=discovery_cisco_wlc_temp,
    check_function=check_cisco_wlc_temp,
    check_default_parameters={'levels_upper': (36, 38), 'levels_lower': (12, 10)},
    check_ruleset_name='cisco_wlc_temp',
)
